!MNH_LIC Copyright 2011-2021 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
!     ###################
      MODULE MODD_MEAN_FIELD_n
!     ###################
!
!!****  *MODD_MEAN_FIELD$n* - declaration of mean variables
!!
!!    PURPOSE
!!    -------
!       The purpose of this declarative module is to specify  the 
!     mean variables. 
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!      
!!
!!    AUTHOR
!!    ------
!!	P.Aumond     *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       01/07/11                      
!!                      10/2016 (C.Lac) Add max values
!!                      12/2021 (R. Schoetter) adds humidity and other mean diagnostics
!!                      11/2022 (E. Jezequel) add covariances
!!
!-------------------------------------------------------------------------------
!
!*       0.   DECLARATIONS
!             ------------
!
USE MODD_PARAMETERS, ONLY: JPMODELMAX
USE MODD_MEAN_FIELD
IMPLICIT NONE

TYPE MEAN_FIELD_t
  REAL, DIMENSION(:,:,:), POINTER :: XUM_MEAN=>NULL(),XVM_MEAN=>NULL(),XWM_MEAN=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XTHM_MEAN=>NULL()     
  REAL, DIMENSION(:,:,:), POINTER :: XTEMPM_MEAN=>NULL()  
  REAL, DIMENSION(:,:,:), POINTER :: XTKEM_MEAN=>NULL()   
  REAL, DIMENSION(:,:,:), POINTER :: XPABSM_MEAN=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XQ_MEAN=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XRH_W_MEAN=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XRH_I_MEAN=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XRH_P_MEAN=>NULL()
  REAL, DIMENSION(:,:)  , POINTER :: XRH_W_MAXCOL_MEAN=>NULL()
  REAL, DIMENSION(:,:)  , POINTER :: XRH_I_MAXCOL_MEAN=>NULL()
  REAL, DIMENSION(:,:)  , POINTER :: XRH_P_MAXCOL_MEAN=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XWIFF_MEAN=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XWIDD_MEAN=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XWIFF_MAX=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XWIDD_MAX=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XSVT_MEAN=>NULL()

  REAL, DIMENSION(:,:,:), POINTER :: XU2_M2=>NULL(),XV2_M2=>NULL(),XW2_M2=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XTH2_M2=>NULL()      
  REAL, DIMENSION(:,:,:), POINTER :: XTEMP2_M2=>NULL() 
  REAL, DIMENSION(:,:,:), POINTER :: XPABS2_M2=>NULL()  

  REAL, DIMENSION(:,:,:), POINTER :: XUV_MEAN=>NULL(),XUW_MEAN=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XVW_MEAN=>NULL(),XWTH_MEAN=>NULL()
  
  REAL, DIMENSION(:,:,:), POINTER :: XUM_MAX=>NULL(),XVM_MAX=>NULL(),XWM_MAX=>NULL()
  REAL, DIMENSION(:,:,:), POINTER :: XTHM_MAX=>NULL()     
  REAL, DIMENSION(:,:,:), POINTER :: XTEMPM_MAX=>NULL()  
  REAL, DIMENSION(:,:,:), POINTER :: XTKEM_MAX=>NULL()   
  REAL, DIMENSION(:,:,:), POINTER :: XPABSM_MAX=>NULL()

          
  INTEGER :: MEAN_COUNT

 !
END TYPE MEAN_FIELD_t

TYPE(MEAN_FIELD_t), DIMENSION(JPMODELMAX), TARGET, SAVE :: MEAN_FIELD_MODEL

REAL, DIMENSION(:,:,:), POINTER :: XUM_MEAN=>NULL(),XVM_MEAN=>NULL(),XWM_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XTHM_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XTEMPM_MEAN=>NULL() 
REAL, DIMENSION(:,:,:), POINTER :: XTKEM_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XPABSM_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XQ_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XRH_W_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XRH_I_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XRH_P_MEAN=>NULL()
REAL, DIMENSION(:,:)  , POINTER :: XRH_W_MAXCOL_MEAN=>NULL()
REAL, DIMENSION(:,:)  , POINTER :: XRH_I_MAXCOL_MEAN=>NULL()
REAL, DIMENSION(:,:)  , POINTER :: XRH_P_MAXCOL_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XWIFF_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XWIDD_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XWIFF_MAX=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XWIDD_MAX=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XSVT_MEAN=>NULL()

REAL, DIMENSION(:,:,:), POINTER :: XU2_M2=>NULL(),XV2_M2=>NULL(),XW2_M2=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XTH2_M2=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XTEMP2_M2=>NULL() 
REAL, DIMENSION(:,:,:), POINTER :: XPABS2_M2=>NULL()

REAL, DIMENSION(:,:,:), POINTER :: XUV_MEAN=>NULL(),XUW_MEAN=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XVW_MEAN=>NULL(),XWTH_MEAN=>NULL()

REAL, DIMENSION(:,:,:), POINTER :: XUM_MAX=>NULL(),XVM_MAX=>NULL(),XWM_MAX=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XTHM_MAX=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XTEMPM_MAX=>NULL() 
REAL, DIMENSION(:,:,:), POINTER :: XTKEM_MAX=>NULL()
REAL, DIMENSION(:,:,:), POINTER :: XPABSM_MAX=>NULL()

INTEGER, POINTER :: MEAN_COUNT =>NULL()

CONTAINS

SUBROUTINE MEAN_FIELD_GOTO_MODEL(KFROM, KTO)
INTEGER, INTENT(IN) :: KFROM, KTO
!
! Save current state for allocated arrays
MEAN_FIELD_MODEL(KFROM)%XUM_MEAN=>XUM_MEAN
MEAN_FIELD_MODEL(KFROM)%XVM_MEAN=>XVM_MEAN
MEAN_FIELD_MODEL(KFROM)%XWM_MEAN=>XWM_MEAN
MEAN_FIELD_MODEL(KFROM)%XTHM_MEAN=>XTHM_MEAN
MEAN_FIELD_MODEL(KFROM)%XTEMPM_MEAN=>XTEMPM_MEAN
MEAN_FIELD_MODEL(KFROM)%XTKEM_MEAN=>XTKEM_MEAN
MEAN_FIELD_MODEL(KFROM)%XPABSM_MEAN=>XPABSM_MEAN
MEAN_FIELD_MODEL(KFROM)%XQ_MEAN=>XQ_MEAN
MEAN_FIELD_MODEL(KFROM)%XRH_W_MEAN=>XRH_W_MEAN
MEAN_FIELD_MODEL(KFROM)%XRH_I_MEAN=>XRH_I_MEAN
MEAN_FIELD_MODEL(KFROM)%XRH_P_MEAN=>XRH_P_MEAN
MEAN_FIELD_MODEL(KFROM)%XRH_W_MAXCOL_MEAN=>XRH_W_MAXCOL_MEAN
MEAN_FIELD_MODEL(KFROM)%XRH_I_MAXCOL_MEAN=>XRH_I_MAXCOL_MEAN
MEAN_FIELD_MODEL(KFROM)%XRH_P_MAXCOL_MEAN=>XRH_P_MAXCOL_MEAN
MEAN_FIELD_MODEL(KFROM)%XWIFF_MEAN=>XWIFF_MEAN
MEAN_FIELD_MODEL(KFROM)%XWIDD_MEAN=>XWIDD_MEAN
MEAN_FIELD_MODEL(KFROM)%XWIFF_MAX=>XWIFF_MAX
MEAN_FIELD_MODEL(KFROM)%XWIDD_MAX=>XWIDD_MAX
MEAN_FIELD_MODEL(KFROM)%XSVT_MEAN=>XSVT_MEAN

MEAN_FIELD_MODEL(KFROM)%XUM_MAX=>XUM_MAX
MEAN_FIELD_MODEL(KFROM)%XVM_MAX=>XVM_MAX
MEAN_FIELD_MODEL(KFROM)%XWM_MAX=>XWM_MAX
MEAN_FIELD_MODEL(KFROM)%XTHM_MAX=>XTHM_MAX
MEAN_FIELD_MODEL(KFROM)%XTEMPM_MAX=>XTEMPM_MAX
MEAN_FIELD_MODEL(KFROM)%XTKEM_MAX=>XTKEM_MAX
MEAN_FIELD_MODEL(KFROM)%XPABSM_MAX=>XPABSM_MAX

MEAN_FIELD_MODEL(KFROM)%XU2_M2=>XU2_M2
MEAN_FIELD_MODEL(KFROM)%XV2_M2=>XV2_M2
MEAN_FIELD_MODEL(KFROM)%XW2_M2=>XW2_M2
MEAN_FIELD_MODEL(KFROM)%XTH2_M2=>XTH2_M2
MEAN_FIELD_MODEL(KFROM)%XTEMP2_M2=>XTEMP2_M2
MEAN_FIELD_MODEL(KFROM)%XPABS2_M2=>XPABS2_M2

IF (LCOV_FIELD) THEN
  MEAN_FIELD_MODEL(KFROM)%XUV_MEAN=>XUV_MEAN
  MEAN_FIELD_MODEL(KFROM)%XUW_MEAN=>XUW_MEAN
  MEAN_FIELD_MODEL(KFROM)%XVW_MEAN=>XVW_MEAN
  MEAN_FIELD_MODEL(KFROM)%XWTH_MEAN=>XWTH_MEAN
END IF
!
! Current model is set to model KTO
XUM_MEAN=>MEAN_FIELD_MODEL(KTO)%XUM_MEAN
XVM_MEAN=>MEAN_FIELD_MODEL(KTO)%XVM_MEAN
XWM_MEAN=>MEAN_FIELD_MODEL(KTO)%XWM_MEAN
XTHM_MEAN=>MEAN_FIELD_MODEL(KTO)%XTHM_MEAN
XTEMPM_MEAN=>MEAN_FIELD_MODEL(KTO)%XTEMPM_MEAN
XTKEM_MEAN=>MEAN_FIELD_MODEL(KTO)%XTKEM_MEAN
XPABSM_MEAN=>MEAN_FIELD_MODEL(KTO)%XPABSM_MEAN
XQ_MEAN=>MEAN_FIELD_MODEL(KTO)%XQ_MEAN
XRH_W_MEAN=>MEAN_FIELD_MODEL(KTO)%XRH_W_MEAN
XRH_I_MEAN=>MEAN_FIELD_MODEL(KTO)%XRH_I_MEAN
XRH_P_MEAN=>MEAN_FIELD_MODEL(KTO)%XRH_P_MEAN
XRH_W_MAXCOL_MEAN=>MEAN_FIELD_MODEL(KTO)%XRH_W_MAXCOL_MEAN
XRH_I_MAXCOL_MEAN=>MEAN_FIELD_MODEL(KTO)%XRH_I_MAXCOL_MEAN
XRH_P_MAXCOL_MEAN=>MEAN_FIELD_MODEL(KTO)%XRH_P_MAXCOL_MEAN
XWIFF_MEAN=>MEAN_FIELD_MODEL(KTO)%XWIFF_MEAN
XWIDD_MEAN=>MEAN_FIELD_MODEL(KTO)%XWIDD_MEAN
XWIFF_MAX=>MEAN_FIELD_MODEL(KTO)%XWIFF_MAX
XWIDD_MAX=>MEAN_FIELD_MODEL(KTO)%XWIDD_MAX
XSVT_MEAN=>MEAN_FIELD_MODEL(KTO)%XSVT_MEAN

XUM_MAX=>MEAN_FIELD_MODEL(KTO)%XUM_MAX
XVM_MAX=>MEAN_FIELD_MODEL(KTO)%XVM_MAX
XWM_MAX=>MEAN_FIELD_MODEL(KTO)%XWM_MAX
XTHM_MAX=>MEAN_FIELD_MODEL(KTO)%XTHM_MAX
XTEMPM_MAX=>MEAN_FIELD_MODEL(KTO)%XTEMPM_MAX
XTKEM_MAX=>MEAN_FIELD_MODEL(KTO)%XTKEM_MAX
XPABSM_MAX=>MEAN_FIELD_MODEL(KTO)%XPABSM_MAX

XU2_M2=>MEAN_FIELD_MODEL(KTO)%XU2_M2
XV2_M2=>MEAN_FIELD_MODEL(KTO)%XV2_M2
XW2_M2=>MEAN_FIELD_MODEL(KTO)%XW2_M2
XTH2_M2=>MEAN_FIELD_MODEL(KTO)%XTH2_M2
XTEMP2_M2=>MEAN_FIELD_MODEL(KTO)%XTEMP2_M2
XPABS2_M2=>MEAN_FIELD_MODEL(KTO)%XPABS2_M2

IF (LCOV_FIELD) THEN
  XUV_MEAN=>MEAN_FIELD_MODEL(KTO)%XUV_MEAN
  XUW_MEAN=>MEAN_FIELD_MODEL(KTO)%XUW_MEAN
  XVW_MEAN=>MEAN_FIELD_MODEL(KTO)%XVW_MEAN
  XWTH_MEAN=>MEAN_FIELD_MODEL(KTO)%XWTH_MEAN
END IF

MEAN_COUNT=>MEAN_FIELD_MODEL(KTO)%MEAN_COUNT

END SUBROUTINE MEAN_FIELD_GOTO_MODEL

END MODULE MODD_MEAN_FIELD_n
